/*
 * This software is the proprietary information of nico3000.
 */
package controls;

import javax.vecmath.Vector3f;

import logging.Logger;

import org.lwjgl.input.Mouse;

import process.EngineProcess;
import util.MathUtil;
import actor.Actor;
import actor.components.TransformComponent;

/**
 *
 * @author nico3000
 */
public class FpsControlsProcess extends EngineProcess {
    private Actor actor;
    private TransformComponent comp;
    private final Vector3f move = new Vector3f();
    private final Vector3f initialOrientation = new Vector3f();
    private final Vector3f deltaOrientation = new Vector3f();
    private float speedModifier = 1.0f;
    private boolean orientationChanged = true;

    @Override
    protected void onUpdate(long dMillis) {
        if(this.actor != null) {
            float speed = this.speedModifier * (float)dMillis * 1e-3f;
            if(this.move.lengthSquared() != 0 && speed != 0) {
                this.comp.getPose().moveRelative(speed * this.move.z, speed * this.move.x, speed * this.move.y, true);
            }
            if(this.orientationChanged) {
            	float xRot = this.initialOrientation.x + this.deltaOrientation.x;
            	float yRot = this.initialOrientation.y + this.deltaOrientation.y;
            	float zRot = this.initialOrientation.z + this.deltaOrientation.z;
                this.comp.getPose().setRotation(xRot, yRot, zRot);
//                Logger.INSTANCE.info("Set Rotation to " + xRot + ", " + yRot + ", " + zRot);
                this.orientationChanged = false;
            }
        }
    }
    
//    public Vector3f getViewDir() {
//    	float xRot = this.initialOrientation.x + this.deltaOrientation.x;
//    	float yRot = this.initialOrientation.y + this.deltaOrientation.y;
//    	float zRot = this.initialOrientation.z + this.deltaOrientation.z;
//    	
//    	
//    }
    
    public void setActor(Actor actor) {
        this.deltaOrientation.set(0.0f, 0.0f, 0.0f);
        this.initialOrientation.set(0.0f, 0.0f, 0.0f);
        this.actor = actor;
        this.comp = null;
        if(this.actor != null) {
            this.comp = (TransformComponent)this.actor.getComponent(TransformComponent.ID);
            if(this.comp == null) {
                this.actor = null;
            } else {
            	Vector3f rotation = new Vector3f();
            	MathUtil.lwjgl2vecmath(rotation, this.comp.getPose().getRotationVector());
//            	Logger.INSTANCE.info("INITIAL CAM POSE : " + rotation.toString());
            	this.initialOrientation.set(rotation);
            	this.orientationChanged = true;
            }
        }
    }
    
    public Vector3f getMoveVector() {
        return this.move;
    }
    
    public void setOrientation(float dYaw, float dPitch, float dRoll) {
        if(Mouse.isGrabbed()) {
//            float newYaw = (this.deltaOrientation.x + dYaw) % MathUtil.PI_MUL2;
//            float newPitch = MathUtil.clamp(this.deltaOrientation.y + dPitch, -MathUtil.PI_DIV2, +MathUtil.PI_DIV2);
            float newYaw = (this.deltaOrientation.x + dYaw) % MathUtil.PI_MUL2;
            float newPitch = this.deltaOrientation.y + dPitch % MathUtil.PI;
            this.orientationChanged = newYaw != this.deltaOrientation.x || newPitch != this.deltaOrientation.y || dRoll != 0.0f;
            this.deltaOrientation.x = newYaw;
            this.deltaOrientation.y = newPitch;
            this.deltaOrientation.z += dRoll;
        }
    }

    public void setSpeedModifier(float speedModifier) {
        this.speedModifier = speedModifier;
    }
}
