/*
 * This software is the proprietary information of nico3000.
 */
package event;

import java.util.LinkedList;
import main.GameApp;
import process.EngineProcess;

/**
 *
 * @author nico3000
 */
public class EventRecorder extends EngineProcess {
    /**
     * Moegliche Stati des Recorders.
     */
    private enum Status {
        /**
         * Zurzeit wird weder aufgenommen noch abgespielt.
         */
        STOPPED,
        /**
         * Aufnahme laeuft.
         */
        RECORDING,
        /**
         * Events werden abgespielt.
         */
        PLAYING,
    }
    
    /**
     * Fortschritt der aktuellen Wiedergabe.
     */
    private long time = 0;
    /**
     * Der aktuelle Status.
     */
    private Status status = Status.STOPPED;
    /**
     * Die aufgenommenen Events.
     */
    private LinkedList<EventData> records = new LinkedList<>();
    /**
     * Das naechste Event, dass abgespielt wird.
     */
    private int pos = 0;
    
    /**
     * Startet eine neue Aufnahme, falls der Recorder gerade gestoppt ist.
     * @return <tt>true</tt>, falls eine neue Aufnahme gestartet wurde
     */
    public boolean startRecording() {
        if(this.status == Status.STOPPED) {
            this.status = Status.RECORDING;
            this.records.clear();
            this.time = GameApp.getRealTimer().getTimeStamp();
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Spiel die letzte Aufnahme ab, sofer der Recorder gestoppt ist.
     * @return <tt>true</tt>, falls die Aufnahme abspielt werden kann
     */
    public boolean playRecordings() {
        if(this.status == Status.STOPPED) {
            this.pos = 0;
            this.time = this.records.isEmpty() ? 0 : this.records.getFirst().getTimeStamp();
            this.status = Status.PLAYING;
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Registriert ein Event, falls der Recorder am Aufnahmen ist.
     * @param data Das Event
     * @return <tt>true</tt>, falls das Event erfolgreich aufgenommen wurde
     */
    public boolean record(EventData data) {
        if(this.status == Status.RECORDING) {
            this.records.add(data);
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Stoppt die Aufnahme bzw. Wiedergabe.
     */
    public void stop() {
        this.status = Status.STOPPED;
    }
    
    /**
     * Prueft, ob der Recorder gerade aufnimmt.
     * @return <tt>true</tt>, falls er aufnimmt
     */
    public boolean isRecording() {
        return this.status == Status.RECORDING;
    }
    
    /**
     * Prueft, ob der Recorder gerade eine Aufnahme abspielt.
     * @return <tt>true</tt>, falls er eine Aufnahme abspielt
     */    
    public boolean isPlaying() {
        return this.status == Status.PLAYING;
    }

    @Override
    protected void onUpdate(long dMillis) {
        if(this.status == Status.PLAYING) {
            if(this.pos >= this.records.size()) {
                this.status = Status.STOPPED;
            } else {
                EventData evt = this.records.get(this.pos);
                if(this.time >= evt.getTimeStamp()) {
                    GameApp.getEventManager().queueEvent(evt);
                    ++this.pos;
                }
            }
            this.time += dMillis;
        }
    }
}
