/*
 * This software is the proprietary information of nico3000.
 */
package gui.components;

import static gui.components.Component.REPAINT_ON_NULL_TEXTURE;
import java.awt.Rectangle;

/**
 *
 * @author nico3000
 */
public class Window extends Component {
    private String title;
    private boolean dragging = false;
    private boolean closeable;
    private final Rectangle titleBarRect = new Rectangle();
    private final Rectangle closeButtonRect = new Rectangle();
    private boolean buttonPressed = false;
    private boolean buttonHovered = false;

    public Window(String title, boolean closeable) {
        super(REPAINT_ON_NULL_TEXTURE | REPAINT_ON_RESIZE | REPAINT_ON_FOCUS_GAINED | REPAINT_ON_FOCUS_LOST);
        this.title = title;
        this.closeable = closeable;
    }

    @Override
    protected void repaint() {
        this.setTexture(this.getGui().getDrawer().createWindow(this.title, this.titleBarRect, this.closeButtonRect, this.isFocused(), this.closeable, this.buttonHovered && this.buttonPressed, this.getWidth(), this.getHeight()));
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        this.updateTitleBarRect();
        this.needsRepaint();
    }

    @Override
    public void setAbsolutePosition(int x, int y) {
        super.setAbsolutePosition(x, y);
        this.updateTitleBarRect();
    }

    @Override
    public void onCursorMoved(int posX, int posY, int dX, int dY) {
        boolean wasHovered = this.buttonHovered;
        this.buttonHovered = this.closeButtonRect.contains(this.getGui().getCursor().getPointerX() - this.getGlobalX(), this.getGui().getCursor().getPointerY() - this.getGlobalY());
        if(this.buttonHovered != wasHovered) {
            this.needsRepaint();
        }
        if(this.dragging) {
            this.setAbsolutePosition(this.getGlobalX() + dX, this.getGlobalY() + dY);
        }
    }

    @Override
    public void onButtonDown(int button, int x, int y) {
        if(button == 0) {
            if(this.titleBarRect.contains(x - this.getGlobalX(), y - this.getGlobalY())) {
                this.dragging = true;
            } else if(this.closeable && this.closeButtonRect.contains(x - this.getGlobalX(), y - this.getGlobalY())) {
                this.buttonPressed = true;
                this.needsRepaint();
            }
        }
    }

    @Override
    public void onButtonUp(int button, int x, int y, boolean clicked) {
        if(button == 0) {
            this.dragging = false;
            this.buttonPressed = false;
            if(this.closeable && this.closeButtonRect.contains(x - this.getGlobalX(), y - this.getGlobalY())) {
                this.hide();
            }
            this.needsRepaint();
        }
    }
    
    private void updateTitleBarRect() {
        this.titleBarRect.setBounds(0, this.getHeight() - 24, this.getWidth() - 24, 24);
        this.closeButtonRect.setBounds(this.getWidth() - 24, this.getHeight() - 24, 20, 20);
    }
}
