/*
 * This software is the proprietary information of nico3000.
 */
package mesh.loader;

import graphics.geometry.Geometry;
import org.w3c.dom.Node;

/**
 * Interface fuer das Laden der Geometriedaten eines Meshes.
 * @author nico3000
 */
public interface GeometryLoader {
    /**
     * Ermoeglicht das Anpassen der Ladeeinstellungen.
     * @param data Der Knoten, der das Mesh eines Actors definiert
     */
    public void setAdditionalData(Node data);
    
    /**
     * Erzeugt eine Geometry.
     * @return Die erzeugte Geometry oder <tt>null</tt>, falls ein Fehler
     *         aufgetreten ist
     */
    public Geometry load();
    
    /**
     * Liefert eine benutzerdefinierte MaterialLibrary fuer das Mesh. Die
     * Information sollte durch die Methode <tt>setAdditionalData()</tt>
     * uebergeben werden.
     * @return Die benutzerdefinierte MaterialLibrary
     */
    public String getCustomMaterialLibrary();
    
    /**
     * Gibt alle benutzten Daten wieder frei. Sollte ganz zum Schluss aufgerufen
     * werden.
     */
    public void release();
}
