/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package resource.file;

import resource.Resource;

/**
 * Eine ResourceFile stellt einer Sammlung von mehreren Resourcen dar.
 * @author nico3000
 */
public interface ResourceFile {
    /**
     * Oeffnet die Datei zum Lesen.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean open();
    
    /**
     * Schliesst die Datei.
     */
    public void close();
    
    /**
     * Liefert die Groesse in Byte einer Resource in dieser Datei.
     * @param resource Die Resource
     * @return Die Groesse der Resource in Byte
     */
    public int getRawResourceSize(Resource resource);
    
    /**
     * Liest eine Resource und speichert den Inhalt in einen Buffer.
     * @param resource Die Resource
     * @param buffer Der Buffer, in den geschrieben werden soll
     * @return Die Anzahl der gelesenen Byte
     */
    public int getRawResource(Resource resource, byte buffer[]);
    
    /**
     * Liefert die Anzahl der in dieser Datei vorhandenen Resourcen.
     * @return Die Anzahl der Resourcen
     */
    public int getNumResources();
    
    /**
     * Liefert den Namen der n-ten Resource.
     * @param num Die Nummer der Resource
     * @return Der Name der Resource
     */
    public String getResourceName(int num);
}
