package scripting;

import java.util.logging.Level;

import logging.Logger;
import main.GameApp;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.EcmaError;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.ImporterTopLevel;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.UniqueTag;

import resource.ResHandle;
import resource.Resource;

public class JavaScriptManager implements ScriptManager {

	private Context context;
	private Scriptable scope;
	
	
	@Override
	public boolean init() {
		
		context = Context.enter();
		scope = new ImporterTopLevel(context);
		
		Logger.INSTANCE.info("Initialized JavaScriptManager for JS 1.7");
		
		return true;
	}

	@Override
	public void destroy() {
		context.exit();
	}

	@Override
	public void executeFile(String resource) {
		Resource file = new Resource(resource);
        ResHandle handle = GameApp.getResCache().getHandle(file);
        if(handle != null) {
            String src = new String(handle.getBuffer());
            handle.unlock();
            // delete resource explicitly to allow reloading while program is
            // running
            GameApp.getResCache().deleteResource(file);
            this.executeString(resource, src);
        }
	}

	@Override
	public void executeString(String fileName, String source) {
		try {
			context.evaluateString(scope, source, fileName, 1, null);
		} catch (EvaluatorException ex) {
            Logger.INSTANCE.error("JavaScriptManager: " + ex.getMessage());
        } catch (EcmaError ex) {
        	Logger.INSTANCE.error("JavaScriptManager: " + ex.getMessage());
        }
//		Logger.INSTANCE.info("Executed JS File: " + fileName);
	}

	@Override
	public Object invokeFunction(String name, Object... args) {
        Object result = null;
        
        try {
            Object o = scope.get(name, scope);            
            if(o instanceof Function) {
                Function f = (Function)o;
                if (args == null) {
                    result = f.call(context, scope, scope, new Object[0]);
                } else {
                    result = f.call(context, scope, scope, args);
                }
            } else if(o instanceof UniqueTag) {
                UniqueTag t = (UniqueTag)o;
                Logger.INSTANCE.error(t == UniqueTag.NOT_FOUND ? "JSFunction '" + name + "' not found!" : t.toString());
            }
            
        } catch (EcmaError error) {
            Logger.INSTANCE.error("JS: " + error.getErrorMessage());
        }
        
        return result;
	}

	@Override
	public String getStringValue(String name, Object obj) {
		return scope.get(name, (Scriptable)obj).toString();
	}

}
