/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package scripting;

import java.nio.FloatBuffer;

import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import logging.Logger;
import main.GameApp;

import org.luaj.vm2.LuaError;
import org.luaj.vm2.LuaFunction;
import org.luaj.vm2.LuaValue;
import org.luaj.vm2.lib.jse.CoerceJavaToLua;

import resource.ResHandle;
import resource.Resource;

/**
 *
 * @author nico3000
 */
public class LuaManager implements ScriptManager {
    private ScriptEngine engine;
    private ScriptEngineFactory factory;
    
    @Override
    public boolean init() {
        ScriptEngineManager sem = new ScriptEngineManager();
        this.engine = sem.getEngineByExtension(".lua");
        this.factory = engine.getFactory();
        Logger.INSTANCE.info("Using " + this.factory.getLanguageName() + " " + this.factory.getLanguageVersion() + " for scripting");
        this.engine.put("testVar", FloatBuffer.wrap(new float[] { 3.141f, 2.7f }));
        this.engine.put("EvtMgr", GameApp.getEventManager());
        this.engine.put("GameLogic", GameApp.getGameLogic());
        this.engine.put("name", "Weihnachtsmann");
        return true;
    }

    @Override
    public void destroy() {
        this.factory = null;
        this.engine = null;
    }

    @Override
    public void executeFile(String resource) {
        ResHandle handle = GameApp.getResCache().getHandle(new Resource(resource));
        if(handle != null) {
            String src = new String(handle.getBuffer());
            handle.unlock();
            // delete resource explicitly to allow reloading while program is
            // running
            GameApp.getResCache().deleteResource(new Resource(resource));
            this.executeString(resource, src);
        }
    }

    @Override
    public void executeString(String fileName, String source) {
        try {
            this.engine.eval(source);
        } catch (ScriptException ex) {
            Logger.INSTANCE.error(ex.getFileName() + ", " + ex.getLineNumber() + "(" + ex.getColumnNumber() + "):" + ex.getMessage());
        } catch(LuaError error) {
            Logger.INSTANCE.error(error.getMessage());
        }
    }
    @Override
    public Object invokeFunction(String name, Object ... args) {
    	LuaFunction func = (LuaFunction)this.engine.getContext().getAttribute(name);
		LuaValue arguments[] = new LuaValue[args.length];
		for (int i = 0; i < args.length; i++) {
			Object currArgument = args[i];
			arguments[i] = CoerceJavaToLua.coerce(currArgument);
		}
		func.invoke(arguments);
		return null;
    }

	@Override
	public String getStringValue(String name, Object obj) {
		// TODO Auto-generated method stub
		return null;
	}
    
}
