/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package util;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author nico3000
 */
public class PieDiagram {
    private static final Color colors[] = {
        new Color(34, 177, 76),
        new Color(0, 162, 232),
        new Color(255, 127, 39),
        new Color(63, 72, 204),
        new Color(237, 28, 36),
        new Color(163, 73, 164),
        new Color(127, 127, 127),
    };
    private final BufferedImage image;
    private final Map<String, Float> values = new HashMap<>();

    public PieDiagram(int width, int height) {
        this.image = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
    }
    
    public void addValue(String name, float value) {
        this.values.put(name, value);
    }
    
    public void createImage() {
        float sum = 0.0f;
        for(float value : this.values.values()) {
            sum += value;
        }
        
        Graphics g = this.image.getGraphics();
        g.setColor(new Color(0, 0, 0, 0));
        g.fillRect(0, 0, this.image.getWidth(), this.image.getHeight());
        g.setFont(new Font("Consolas", Font.PLAIN, 20));
        
        int currentAngle = 0;
        int i = 0;
        for(String name : this.values.keySet()) {
            float value = this.values.get(name);
            g.setColor(colors[i % colors.length]);
            int angle = i == values.size() - 1 ? 360 - currentAngle : (int)(360.0f * value / sum);
            g.fillArc(0, 0, this.image.getWidth(), this.image.getHeight(), currentAngle, angle);
            g.drawString(name, 20, this.image.getHeight() - 20 * (i+1));
            currentAngle += angle;
            ++i;
        }
    }

    public BufferedImage getImage() {
        return image;
    }
}
